#!/usr/bin/perl

=head1 NAME

s4pm_find_by_coverage.pl - find data based on full coverage of processing time

=head1 SYNOPSIS

s4pm_find_by_coverage.pl 
I<config_parms>

=head1 DESCRIPTION

This script is meant to work with the s4pm_find_data.pl script of the Find
Data station via the -find option. This option allows a custom script to 
locate desired data rather than having the s4pm_find_data.pl script do it.

This find script examines all metadata files in a data directory and examines
the beginning and ending date/time to see which completely covers the
processing period. If one is found, it immediately exits with that path
name. If none are found, it exits with no return value which Find Data
will interpret as a null result.

=head1 AUTHOR

Stephen Berrick, NASA/GSFC, Code 610.2

=cut

# s4pm_find_by_coverage.pl,v 1.4 2006/10/10 15:23:20 sberrick Exp
# -@@@ S4PM, Version Release-5_16_0
################################################################################

use Safe;
use S4P::MetFile;
use S4P::TimeTools;

my $config = $ARGV[0];

my $compartment = new Safe 'CFG';
$compartment->share('$data_dir', '$process_start', '$process_stop');
$compartment->rdo($config) or return "Failed to read $config in safe mode.";

my @metfiles = glob("$CFG::data_dir/*.met");
foreach my $met ( @metfiles ) {
    my ($begin, $end) = S4P::MetFile::get_spatial_temporal_metadata($met);
    my $res1 = S4P::TimeTools::CCSDSa_DateCompare($begin, $CFG::process_start);
    my $res2 = S4P::TimeTools::CCSDSa_DateCompare($end, $CFG::process_stop);
    unless ( $res1 == -1 or $res2 == +1 ) { 
        $path = $met;
        $path =~ s/\.met$//;
        print $path;
        exit 0;
    }
}

exit 0;
